
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: AccumulationTraits.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Definitions of the accumulation traits for images.	
//
// ==========================================================================


#ifndef BoxySoft_AccumulationTraits
#define BoxySoft_AccumulationTraits






///////////////////////////////////////////////////////////////////////////////////////////////////

// This trait defines a type which
// will be used when accumulating 
// data of different types.
template < typename T >
class Accumulation_Trait;


// Specific traits are implemented 
// as specializations.
template <>
class Accumulation_Trait< double >
{
	public: 
		// For double, accumulator is: long double
		typedef long double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0.0; }
};

template <>
class Accumulation_Trait< float >
{
	public: 
		// For float, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0.0; }
};

template <>
class Accumulation_Trait< long >
{
	public: 
		// For long, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0.0; }
};

template <>
class Accumulation_Trait< int >
{
	public: 
		// For int, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0.0; }
};

template <>
class Accumulation_Trait< short >
{
	public: 
		// For short, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0.0; }
};

template <>
class Accumulation_Trait< char >
{
	public: 
		// For char, accumulator is: long
		typedef long TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0; }
};


template <>
class Accumulation_Trait< unsigned long >
{
	public: 
		// For unsigned long, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0; }
};

template <>
class Accumulation_Trait< unsigned int >
{
	public: 
		// For unsigned int, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0; }
};

template <>
class Accumulation_Trait< unsigned short >
{
	public: 
		// For unsigned short, accumulator is: double
		typedef double TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0; }
};

template <>
class Accumulation_Trait< unsigned char >
{
	public: 
		// For char, accumulator is: long
		typedef unsigned long TheAccumulatorType;
		static TheAccumulatorType GetZero( void ) { return 0; }
};



///////////////////////////////////////////////////////////////////////////////////////////////////

#endif BoxySoft_AccumulationTraits




