
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: ImageMatrix.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Exemplary functions to test image algebra.
//
// ==========================================================================


#include "ImageMatrix.h"
#include <math.h>
#include <time.h>



void Test_Solve_Linear_System( void )
{
	static double A[ 5 ][ 5 ] = { 
									{ 0.0, -17.17, 23, 0, 44 },
									{ -100.0, -1.17, -18.55, 0, 87 },
									{ 15.0, -174, 23, 4.13, 78 },
									{ 33.0, 22.5, 23, 22.22, -7 },
									{ 1.0, -0.1, 23, 5.7, 133 }
	};


	static double B[ 5 ] = { 10.0, 20.0, 13.0, 22.22, -7.17	};

	TRealImage matrix_A( 5, 5, 0.0 );
	memcpy( (unsigned char*)matrix_A.GetImageData(), (unsigned char*)A, 5 * 5 * sizeof( double ) );


	TRealImage matrix_B( 1, 5, 0.0 );
	memcpy( (unsigned char*)matrix_B.GetImageData(), (unsigned char*)B, 1 * 5 * sizeof( double ) );


	TRealImage * out_x = Orphan_Linear_Solution< double, double >( matrix_A, matrix_B );


	Save_ASCII_Image( * out_x, "x.txt", ", " );
	

	// Check it out
	TRealImage * outMult = Orphan_Mult_Matrix< double, double >( matrix_A, * out_x );

	if( outMult != 0 )
	{
		double theTotalSum = 0.0;
		for( int i = 0; i < outMult->GetCol(); ++ i )
			for( int j = 0; j < outMult->GetRow(); ++ j )
				theTotalSum += fabs( outMult->GetPixel( i, j ) - matrix_B.GetPixel( i, j ) );

//			AfxMessageBox( theTotalSum < 0.000001 ? "Linear system solved OK :)" : "Linear system FAILED xx:(x" );
	}

	delete outMult;
	delete out_x;
}


void Test_Solve_Linear_System_2( void )
{
	int size = 200;

	TRealImage matrix_A( size, size, 0.0 );
	TRealImage matrix_B( 1, size, 0.0 );

	// Seed the random-number generator with current time so that
	// the numbers will be different every time we run. 
	srand( (unsigned)time( NULL ) );

	int i;
	for( i = 0; i < size; ++ i )
		for( int j = 0; j < size; ++ j )
			matrix_A.SetPixel( i, j, rand() );

	for( i = 0; i < size; ++ i )
		matrix_B.SetPixel( 0, i, rand() );


	TRealImage * out_x = Orphan_Linear_Solution< double, double >( matrix_A, matrix_B );

	// Check it out
	TRealImage * outMult = Orphan_Mult_Matrix< double, double >( matrix_A, * out_x );

	if( outMult != 0 )
	{
		double theTotalSum = 0.0;
		for( int i = 0; i < outMult->GetCol(); ++ i )
			for( int j = 0; j < outMult->GetRow(); ++ j )
				theTotalSum += fabs( outMult->GetPixel( i, j ) - matrix_B.GetPixel( i, j ) );

//			AfxMessageBox( theTotalSum < 0.000001 ? "Linear system solved OK :)" : "Linear system FAILED xx:(x" );
	}

	delete outMult;
	delete out_x;
}




void TestOdwroc( void )
{
	static double matrix[ 5 ][ 5 ] = { 
											{ 0.0, -17.17, 23, 0, 44 },
											{ -100.0, -1.17, -18.55, 0, 87 },
											{ 15.0, -174, 23, 4.13, 78 },
											{ 33.0, 22.5, 23, 22.22, -7 },
											{ 1.0, -0.1, 23, 5.7, 133 }
	};

	TRealImage in( 5, 5, 0.0 );
	memcpy( (unsigned char*)in.GetImageData(), (unsigned char*)matrix, 5 * 5 * sizeof( double ) );

	TRealImage * outInv = Orphan_Inv_Matrix< double, double >( in );

	if( outInv != 0 )
		Save_ASCII_Image( * outInv, "odwrocona.txt", ", " );

	TRealImage * outMult = Orphan_Mult_Matrix< double, double >( * outInv, in );

	if( outMult != 0 )
		Save_ASCII_Image( * outMult, "odwrocona_spr_1.txt", ", " );


	delete outInv;
	delete outMult;

}

void TestOdwroc2( void )
{
	int size = 1000;

	TRealImage in( size, size, 0.0 );

	// Seed the random-number generator with current time so that
	// the numbers will be different every time we run. 
	srand( (unsigned)time( NULL ) );

	for( int i = 0; i < size; ++ i )
		for( int j = 0; j < size; ++ j )
			in.SetPixel( i, j, rand() );

	TRealImage * outInv = Orphan_Inv_Matrix< double, double >( in );

	if( outInv == 0 )
	{
//		AfxMessageBox( _T("Cannot invert this matrix :(") );
		return;
	}

	TRealImage * outMult = Orphan_Mult_Matrix< double, double >( * outInv, in );

	if( outMult != 0 )
	{
		double theTotalSum = 0.0;
		for( int i = 0; i < size; ++ i )
			for( int j = 0; j < size; ++ j )
				theTotalSum += outMult->GetPixel( i, j );

//			AfxMessageBox( theTotalSum - size * 1.0 < 0.000001 ? "Matrix inverted OK :)" : "Matrix inversion FAILED xx:(x" );
	}

	delete outInv;
	delete outMult;
}

// The pseudo-inverse
void TestOdwroc3( void )
{
	int size = 100;

	const int kRows = 128;
	const int kCols = 100;


	TRealImage in( kCols, kRows, 0.0 );

	// Seed the random-number generator with current time so that
	// the numbers will be different every time we run. 
	srand( (unsigned)time( NULL ) );

	for( int i = 0; i < kCols; ++ i )
		for( int j = 0; j < kRows; ++ j )
			in.SetPixel( i, j, rand() );

	TRealImage * outInv = Orphan_PseudoInv_Matrix< double, double >( in );

	if( outInv == 0 )
	{
//		AfxMessageBox( _T("Cannot invert this matrix :(") );
		return;
	}

	TRealImage * outMult = Orphan_Mult_Matrix< double, double >( * outInv, in );

	if( outMult != 0 )
	{
		double theTotalSum = 0.0;
		for( int i = 0; i < size; ++ i )
			for( int j = 0; j < size; ++ j )
				theTotalSum += outMult->GetPixel( i, j );

//			AfxMessageBox( theTotalSum - size * 1.0 < 0.000001 ? "Matrix pseudo-inverted OK :)" : "Matrix pseudo-inversion FAILED xx:(x" );
	}

	delete outInv;
	delete outMult;
}





