
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: RealLinearFilter.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Classes for linear filters.	
//
// ==========================================================================




#ifndef BoxySoft_RealLinearFilter
#define BoxySoft_RealLinearFilter


#include <memory>
#include "HIL_ImageFor.h"

using namespace PHIL;

///////////////////////////////////////////////////////////
// The base class (pure virtual) for hierarchy of
// linear real filters. 
///////////////////////////////////////////////////////////
class TRealLinearFilter
{
	private:

		// Class inherent variables

	protected:

	public:

		// ===================================================
		TRealLinearFilter( void ) {}	// class default constructor
		virtual ~TRealLinearFilter() {}	// class virtual destructor
		// ===================================================

	public:


		///////////////////////////////////////////////////////////
		// This function does linear filtering of the input image. 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			In - const reference to the input image
		//			Out - reference to the output image
		//		
		// OUTPUT:
		//			none
		//		
		// REMARKS:
		//			See also overloaded versions of this function.
		//		
		virtual void operator() ( const TRealImage & In, TRealImage & Out ) const = 0;

		///////////////////////////////////////////////////////////
		// This function does linear filtering of the input image. 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			In - const reference to the input image
		//		
		// OUTPUT:
		//			ORPHANED filtered image.
		//		
		// REMARKS:
		//			The returned image is orphaned, i.e. the caller
		//			is responsible for its further management and
		//			its final memory de-allocation !!!
		//
		//			See also overloaded versions of this function.
		//		
		virtual TRealImage * operator() ( const TRealImage & In ) const = 0;

		///////////////////////////////////////////////////////////
		// This function does linear filtering of the input image. 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			In - reference to the input image
		//		
		// OUTPUT:
		//			none
		//		
		// REMARKS:
		//			The input image is CHANGED!
		//
		//			See also overloaded versions of this function.
		//		
		virtual void operator() ( TRealImage & In ) const = 0;

};

////////////////////////////////////////////////////////////////////////////

typedef std::auto_ptr< TRealLinearFilter >			RealLinFilter_AP;		

////////////////////////////////////////////////////////////////////////////



#endif BoxySoft_RealLinearFilter




