
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: RealLinearFilter_Factory.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Factory pattern for linear filters.	
//
// ==========================================================================



#ifndef BoxySoft_RealLinearFilter_Factory
#define BoxySoft_RealLinearFilter_Factory


#include "GaussianFilter.h"
#include "BinomialFilter.h"




///////////////////////////////////////////////////////////
// This is abstract factory for real linear filter objects. 
///////////////////////////////////////////////////////////
class TRealLinearFilter_Factory
{
	private:

		// Class inherent variables

	protected:

	public:

		// ===================================================
		TRealLinearFilter_Factory( void ) {}				// class default constructor
		virtual ~TRealLinearFilter_Factory() {}				// class virtual destructor
		// ===================================================

	public:


		///////////////////////////////////////////////////////////
		// This function 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//		
		//		
		// OUTPUT:
		//		
		//		
		// REMARKS:
		//		
		//		
		virtual RealLinFilter_AP operator() ( int mask_width, double sigma ) const = 0;

};

/////////////////////////////////////////////////////////////////////////////////////////

class TGaussianFilter_Factory : public TRealLinearFilter_Factory
{
	private:

		// Class inherent variables

	protected:


	public:

		// ===================================================
		TGaussianFilter_Factory( void ) {}		// class default constructor	
		virtual ~TGaussianFilter_Factory() {}	// class virtual destructor
		// ===================================================

	public:


		///////////////////////////////////////////////////////////
		// This function 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//		
		//		
		// OUTPUT:
		//		
		//		
		// REMARKS:
		//		
		//		
		virtual RealLinFilter_AP operator() ( int mask_width, double sigma ) const
		{
			return RealLinFilter_AP( new TGaussianFilter( mask_width, sigma ) );
		}


	public:

		const static TGaussianFilter_Factory	gkGaussianFilter_Factory_Object;

};

/////////////////////////////////////////////////////////////////////////////////////////

class TBinomialFilter_Factory : public TRealLinearFilter_Factory
{
	private:

		// Class inherent variables

	protected:

	public:

		// ===================================================
		TBinomialFilter_Factory( void ) {}				// class default constructor
		virtual ~TBinomialFilter_Factory() {}			// class virtual destructor
		// ===================================================

	public:


		///////////////////////////////////////////////////////////
		// This function 
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//		
		//		
		// OUTPUT:
		//		
		//		
		// REMARKS:
		//		
		//		
		virtual RealLinFilter_AP operator() ( int mask_width, double sigma ) const
		{
			return RealLinFilter_AP( new TBinomialFilter( mask_width ) );
		}


	public:

		const static TBinomialFilter_Factory	gkBinomialFilter_Factory_Object;

};

/////////////////////////////////////////////////////////////////////////////////////////



#endif BoxySoft_RealLinearFilter_Factory




