
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: HIL_Exceptions.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Implementation of exceptions.
//
// ==========================================================================



#include "HIL_Exceptions.h"


// ==========================================================================
namespace PHIL		
{
// ==========================================================================



const char * T_Standard_HIL_Exception::HIL_Standard_ErrorHandlerMessage[] =
{
	"Standard_HIL_Exception: Unknown error.",					// for kUnknown
	"Standard_HIL_Exception: Divide by zero.",					// for kDivideByZero
	"Standard_HIL_Exception: Wrong input parameters.",			// for kWrongInputParam
	"Standard_HIL_Exception: Wrong output parameters.",			// for kWrongOutputParam
	"Standard_HIL_Exception: Image dimensions mismatch.",		// for kImageDimensionsMismatch
	"Standard_HIL_Exception: Wrong pixel type.",				// for kWrongPixelType
	"Standard_HIL_Exception: Result is infinity.",				// for kResultInfinity

};



///////////////////////////////////////////////////////////
// This function returns a string with description
// of an exception.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			error explanation message for this object
//		
// REMARKS:
//			This function will NOT throw any exceptions.
		//		
const char * T_Standard_HIL_Exception::what() const throw()
{
	REQUIRE( Get_Error_Code() < k_SENTINEL );
	REQUIRE( Get_Error_Code() < sizeof( HIL_Standard_ErrorHandlerMessage ) / sizeof( HIL_Standard_ErrorHandlerMessage[ 0 ] ) );
	
	if( Get_Error_Code() < sizeof( HIL_Standard_ErrorHandlerMessage ) / sizeof( HIL_Standard_ErrorHandlerMessage[ 0 ] ) )
		return HIL_Standard_ErrorHandlerMessage[ Get_Error_Code() ]; 
	else
		return "";			// very serious internal error - immediately check the HIL_Standard_ErrorHandlerMessage
							// and add missing entries 
}



// ==========================================================================
}	// end of the PHIL namespace
// ==========================================================================
