
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: Image_IO.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Image Input/Output functions.
//
// ==========================================================================


#pragma warning(disable:4786)


#include "Image_IO.h"


#if CONNECT_2_OPEN_CV


#include "HIL_ImageFormatConvertOperators.h"



namespace PHIL		
{


///////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////
// Test function creates an interlaced image from a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			ptr to the created and ORPHANED ColorImage
//				object upon success;
//			0 if failure
//		
// REMARKS:
//			The caller is responsible of disposing 
//				the returned object.
//		
NI_Color_3x8_Image * Orphan_NonInterColorImageFrom( const char * fileName )
{
	IplImage * image_1 = 0; 

	// Directly load a TIFF image to the IPL format
	// WARNING - the Intel's library accepts only non-compressed TIFF format !!!
	image_1 = cvLoadImage( fileName, -1 );
	if( image_1 == 0 )
		return 0;			// apparently cannot load that image

	NI_Color_3x8_Image * outImage = new NI_Color_3x8_Image( image_1->width, image_1->height );

	( * FormatConverter_AP( * outImage, * image_1 ) )();		// change format

	cvReleaseImage( & image_1 );

	return outImage;
}

///////////////////////////////////////////////////////////
// Test function saves a color image to a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			theImage - a reference to the image object
//				to be saved
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			true if success,
//			false otherwise
//		
// REMARKS:
//		
bool Save_ColorImageIn( const NI_Color_3x8_Image & theImage, const char * fileName )
{
	// Create a temporary IPL color image
	const int kPixelDepth = 8;		// 8 bits
	const int kChannels = 3;
	IplImage * image_1 = cvCreateImage( cvSize( theImage.GetRefPixel( 0, 0 ).GetCol(), theImage.GetRefPixel( 0, 0 ).GetRow() ), kPixelDepth, kChannels );
	if( image_1 == 0 )
		return false;			// apparently cannot create that image

	( * FormatConverter_AP( * image_1, theImage ) )();		// change format

	int retVal = cvSaveImage( fileName, image_1 );

	cvReleaseImage( & image_1 );

	return retVal == 1;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////
// Test function creates an non-interlaced image from a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			ptr to the created and ORPHANED ColorImage
//				object upon success;
//			0 if failure
//		
// REMARKS:
//			The caller is responsible of disposing 
//				the returned object.
//		
ColorImage * Orphan_ColorImageFrom( const char * fileName )
{
	IplImage * image_1 = 0; 

	// Directly load a TIFF image to the IPL format
	// WARNING - the Intel's library accepts only non-compressed TIFF format !!!
	image_1 = cvLoadImage( fileName, -1 );
	if( image_1 == 0 )
		return 0;			// apparently cannot load that image

	ColorImage * outImage = new ColorImage( image_1->width, image_1->height );

	( * FormatConverter_AP( * outImage, * image_1 ) )();		// change format

	cvReleaseImage( & image_1 );

	return outImage;
}

///////////////////////////////////////////////////////////
// Test function saves a color image to a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			theImage - a reference to the image object
//				to be saved
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			true if success,
//			false otherwise
//		
// REMARKS:
//		
bool Save_ColorImageIn( const ColorImage & theImage, const char * fileName )
{
	// Create a temporary IPL color image
	const int kPixelDepth = 8;		// 8 bits
	const int kChannels = 3;
	IplImage * image_1 = cvCreateImage( cvSize( theImage.GetCol(), theImage.GetRow() ), kPixelDepth, kChannels );
	if( image_1 == 0 )
		return false;			// apparently cannot create that image

	( * FormatConverter_AP( * image_1, theImage ) )();		// change format

	int retVal = cvSaveImage( fileName, image_1 );

	cvReleaseImage( & image_1 );

	return retVal == 1;
}




//////////////////////////////////////////////////////////////////////////////////////////

}	// end of the PHIL namespace


//////////////////////////////////////////////////////////////////////////////////////////



#endif CONNECT_2_OPEN_CV



























