
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: Image_IO.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Image Input/Output functions via the Intel's Open CV.
//
// ==========================================================================


#ifndef BoxySoft_Image_IO_h
#define BoxySoft_Image_IO_h


#include "IntelLIb_ImageConverters.h"			// for definition of the: CONNECT_2_OPEN_CV


#if CONNECT_2_OPEN_CV

#include "HIL_ImageFor.h"
#include "HIL_MultiChannelImageFor.h"


//////////////////////////////////////////////////////////////////////////////////////////



namespace PHIL		
{


///////////////////////////////////////////////////////////
// Test function creates an interlaced image from a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			ptr to the created and ORPHANED ColorImage
//				object upon success;
//			0 if failure
//		
// REMARKS:
//			The caller is responsible of disposing 
//				the returned object.
//		
NI_Color_3x8_Image * Orphan_NonInterColorImageFrom( const char * fileName );


///////////////////////////////////////////////////////////
// Test function saves a color image to a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			theImage - a reference to the image object
//				to be saved
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			true if success,
//			false otherwise
//		
// REMARKS:
//		
bool Save_ColorImageIn( const NI_Color_3x8_Image & theImage, const char * fileName );



///////////////////////////////////////////////////////////
// Test function creates an non-interlaced image from a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			ptr to the created and ORPHANED ColorImage
//				object upon success;
//			0 if failure
//		
// REMARKS:
//			The caller is responsible of disposing 
//				the returned object.
//		
ColorImage * Orphan_ColorImageFrom( const char * fileName );


///////////////////////////////////////////////////////////
// Test function saves a color image to a file.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			theImage - a reference to the image object
//				to be saved
//			fileName - name of a file to be converted 
//				It must be a color and uncompressed format
//		
// OUTPUT:
//			true if success,
//			false otherwise
//		
// REMARKS:
//		
bool Save_ColorImageIn( const ColorImage & theImage, const char * fileName );




//////////////////////////////////////////////////////////////////////////////////////////

}	// end of the PHIL namespace


//////////////////////////////////////////////////////////////////////////////////////////


#endif CONNECT_2_OPEN_CV










#endif BoxySoft_Image_IO_h




