
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: GaussianImagePyramids.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Definitions of the image pyramids.
//
// ==========================================================================



#ifndef BoxySoft_GaussianImagePyramids
#define BoxySoft_GaussianImagePyramids


#include "ImagePyramids.h"
#include "RealLinearFilter_Factory.h"



///////////////////////////////////////////////////////////
// This class implements the Gaussian image pyramid.
///////////////////////////////////////////////////////////
class TGaussianImagePyramids : public TImagePyramids
{
	private:

		// Class inherent variables

	protected:

		const TRealLinearFilter_Factory & fRealLinearFilter_Factory;


	protected:

		int		fMaskWidth;
		double	fSigma;

	public:

		// ===================================================
		TGaussianImagePyramids( int maskWidth, double sigma, const TRealLinearFilter_Factory & theFactory = TGaussianFilter_Factory::gkGaussianFilter_Factory_Object );				// class default constructor
		virtual ~TGaussianImagePyramids();				// class virtual destructor
		// ===================================================


		///////////////////////////////////////////////////////////
		// This function creates an image pyramid from the input
		// image.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			theInputImage - reference to the input image
		//			theOutputPyramid - vector of output images
		//			requiredPyramidLevels - required number of 
		//				the pyramid levels
		//		
		// OUTPUT:
		//			true - if operation successful
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual  bool operator () ( const TRealImage & theInputImage, RealImage_Vector & theOutputPyramid, int requiredPyramidLevels );

		///////////////////////////////////////////////////////////
		// This function creates an image pyramid from the input
		// image.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			theInputImage - reference to the input image
		//			theOutputPyramid - vector of output images
		//			requiredPyramidLevels - required number of 
		//				the pyramid levels
		//		
		// OUTPUT:
		//			true - if operation successful
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		bool operator () ( const TRealImage & theInputImage, Real_Template_Image & theOutputPyramid, int requiredPyramidLevels );

};


///////////////////////////////////////////////////////////
// This class implements the Difference of Gaussians
// image pyramid.
///////////////////////////////////////////////////////////
class TDOGImagePyramids : public TGaussianImagePyramids
{
	private:

		// Class inherent variables

	protected:

		double	fSigma_2;

	public:

		// ===================================================
		TDOGImagePyramids( int maskWidth, double sigma_1, double sigma_2, const TRealLinearFilter_Factory & theFactory = TGaussianFilter_Factory::gkGaussianFilter_Factory_Object );				// class default constructor
		virtual ~TDOGImagePyramids();				// class virtual destructor
		// ===================================================


		///////////////////////////////////////////////////////////
		// This function creates an image pyramid from the input
		// image.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			theInputImage - reference to the input image
		//			theOutputPyramid - vector of output images
		//			requiredPyramidLevels - required number of 
		//				the pyramid levels
		//		
		// OUTPUT:
		//			true - if operation successful
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual  bool operator () ( const TRealImage & theInputImage, RealImage_Vector & theOutputPyramid, int requiredPyramidLevels );

		virtual  bool operator () ( const TRealImage & theInputImage, Real_Template_Image & theOutputPyramid, int requiredPyramidLevels );

};

///////////////////////////////////////////////////////////
// This class implements the Laplacian image pyramid.
///////////////////////////////////////////////////////////
class TLaplacianImagePyramids : public TGaussianImagePyramids
{
	private:

		// Class inherent variables

	protected:


	public:

		// ===================================================
		TLaplacianImagePyramids( int maskWidth, double sigma, const TRealLinearFilter_Factory & theFactory = TGaussianFilter_Factory::gkGaussianFilter_Factory_Object );				// class default constructor
		virtual ~TLaplacianImagePyramids();				// class virtual destructor
		// ===================================================


		///////////////////////////////////////////////////////////
		// This function creates an image pyramid from the input
		// image.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			theInputImage - reference to the input image
		//			theOutputPyramid - vector of output images
		//			requiredPyramidLevels - required number of 
		//				the pyramid levels
		//		
		// OUTPUT:
		//			true - if operation successful
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual  bool operator () ( const TRealImage & theInputImage, RealImage_Vector & theOutputPyramid, int requiredPyramidLevels );

		virtual  bool operator () ( const TRealImage & theInputImage, Real_Template_Image & theOutputPyramid, int requiredPyramidLevels );

		///////////////////////////////////////////////////////////
		// This function recovers original images from the Laplacian
		// pyramid.
		// The Laplacian pyramid allows for image reconstruction
		// quickly from the sequence of images in the Laplacian
		// pyramid by recursively expanding the images and summing
		// them up:
		//
		//			G[ p + 1 ] = L[ p - 1 ] + F2[ G[ p ] ]
		//
		// Even if the interpolation algorithms required to expand
		// the image contain errors, they effect only the Laplacian
		// pyramid because the same algorithm is used twice.
		//
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//		
		//		
		// OUTPUT:
		//		
		//		
		// REMARKS:
		//		
		//		
		bool RecoverGaussian ( const RealImage_Vector & theInputLaplacePyramid, RealImage_Vector & theOutputPyramid );


};




#endif BoxySoft_GaussianImagePyramids







