
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: PolyWarpingParameterCalculator.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Definitions of the polynomial warping classes.
//
// ==========================================================================



#ifndef BoxySoft_PolyWarpingParameterCalculator
#define BoxySoft_PolyWarpingParameterCalculator


#include "HIL_ImageFor.h"
#include "Warping.h"


using namespace PHIL;



///////////////////////////////////////////////////////////
// This function does 2nd order polynomial warping on
// the supplied mono image.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			inMonoImage - reference to the input image
//				which transformed (warped) copy will
//				be returned upon success
//			origImagePoints - a collection of the points
//				selected in the original image space
//			mappedImagePoints - a collection of the points
//				defining new positions (the mapped space)
//		
// OUTPUT:
//			warped image if success
//			0 otherwise
//		
// REMARKS:
//			The returned object is an auto pointer object
//			that contains orphaned object on success
//		
MIAP OrphanPolyWarpedImage( const MonochromeImage & inMonoImage, Int_2D_Point_Vector & origImagePoints, Int_2D_Point_Vector & mappedImagePoints );



///////////////////////////////////////////////////////////
// This function finds coefficients of the 2nd order
// polynomial warping transformation based on the 
// supplied two sets of corresponding points.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			origImagePoints - a collection of the points
//				selected in the original image space
//			mappedImagePoints - a collection of the points
//				defining new positions (the mapped space)
//			thePolyMatrix - the transformation matrix
//				to be filled
//		
// OUTPUT:
//			true if success
//			false otherwise
//		
// REMARKS:
//		
bool DeterminePolynomialTransformParameters( Int_2D_Point_Vector & origImagePoints, Int_2D_Point_Vector & mappedImagePoints, T_2ndOrder_Polynomial_TransformEngine::Order2ndPolyMatrix & thePolyMatrix );






#endif BoxySoft_PolyWarpingParameterCalculator




