
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: HardwareImageLibraryProxy.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				A suite of functions for testing different modules of the
//				library.
//
// ==========================================================================


#pragma warning(disable:4786)


#include "HardwareImageLibraryProxy.h"

#include "HIL_BaseDefinitions.h"
#include "HIL_ArithmeticOperators.h"
#include "HIL_LogicalOperators.h"
#include "HIL_ComposedOperations.h"
#include "HIL_ConvolveOperators.h"
#include "HIL_FindValueInImageOperators.h"
#include "HIL_ImageFormatConvertOperators.h"
#include "HIL_MultiChannelImageFor.h"
#include "HIL_MultiChannelMixedCopy.h"
#include "HIL_BooleanImage.h"
#include "HIL_MaskedImage.h"
#include "HIL_RankFilterOperators.h"

#include "ImageErrorMeasurementLibrary.h"
#include "Morphology.h"
#include "StereoMatcherFor.h"

#include "HIL_InPlaceAllocator.h"
#include "HIL_VideoFor.h"
#include "LogPolarSpace.h"
#include "SavitzkyGolayFilter.h"








using namespace PHIL;



//////////////////////////////////////////////////////////////////////////

class MyCallBack : public TOperationCompletionCallback
{
	public:

		virtual void * operator()( void ) 
		{
			cout << endl << "Operation completed with SUCCESS!" << endl;
			return 0;
		}
};

//////////////////////////////////////////////////////////////////////////

void SG_Filter_Test( void )
{
	TSavitzkyGolayFilter theSGFilter;

	const long kN			= 2;
	const long kIndexFrom	= -1;
	const long kIndexTo		= +1;

	TRealImage * coeffsImage = theSGFilter.Compute_SavGol_Filer( kN, kIndexFrom, kIndexTo );

	// ... tabularize the coefficients


	delete coeffsImage;

}



void VideoTest( void )
{
	MonochromeVideo monoVid;

	monoVid.AttachOrphanedFrame( new MonochromeVideo::FrameImage( 10, 20 ) );

	monoVid.InsertOrphanedFrameAt( 0, new MonochromeVideo::FrameImage( 10, 20 ) );	// attach to the head

	monoVid.RemoveFrames( 1, 1 );
}


void VideoTest_2( void )
{
	typedef unsigned char																			MonoPixel;
	typedef InPlaceAllocatorFor< MonoPixel >														InPlaceAllocatorForMonochrome;
	typedef TVideoFor< MonoPixel, PixelAccess_Trait< MonoPixel >, InPlaceAllocatorForMonochrome >	MonochromeVideoInPlace;

	int imCols = 128;
	int imRows = 256;
	int imImages = 17;

	unsigned long single_image_size = imCols * imRows * sizeof( unsigned char );



	typedef SaveBufferAllocator< MonoPixel > MonochromeVideo_SafeBufferAllocator;
	MonochromeVideo_SafeBufferAllocator	theMonochromeVideo_SafeBufferAllocator( single_image_size, imImages );		// it works like auto_ptr


	MonochromeVideoInPlace monoVid;


	monoVid.AttachOrphanedFrame( new MonochromeVideoInPlace::FrameImage( imCols, imRows, 0, theMonochromeVideo_SafeBufferAllocator.GetNext_InPlaceAllocator() ) );

	monoVid.AttachOrphanedFrame( new MonochromeVideoInPlace::FrameImage( imCols, imRows, 0, theMonochromeVideo_SafeBufferAllocator.GetNext_InPlaceAllocator() ) );


	monoVid.SetPixel( 1, 1, 1, 125 );
	MonochromeVideo::PixelType pixel = monoVid.GetPixel( 1, 1, 1 );


	monoVid.InsertOrphanedFrameAt( 0, new MonochromeVideoInPlace::FrameImage( imCols, imRows, theMonochromeVideo_SafeBufferAllocator.GetNext_InPlaceAllocator() ) );	// attach to the head


	monoVid.RemoveFrames( 1, 1 );

}

void VideoTest_3( void )
{
	int imCols = 128;
	int imRows = 256;
	int imImages = 17;

	//MonochromeVideo * theVideo = OrphanMonochromeVideo( imCols, imRows, imImages );
	MVAP theVideo( OrphanMonochromeVideo( imCols, imRows, imImages ) );


	theVideo->SetPixel( 1, 1, 0, 100 );
	MonochromeVideo::PixelType pixel = theVideo->GetPixel( 1, 1, 0 );

	theVideo->SetPixel( 1, 1, 1, 101 );
	pixel = theVideo->GetPixel( 1, 1, 1 );

	theVideo->SetPixel( 1, 1, 2, 102 );
	pixel = theVideo->GetPixel( 1, 1, 2 );

	theVideo->SetPixel( 1, 1, 3, 103 );
	pixel = theVideo->GetPixel( 1, 1, 3 );

	// ...

	theVideo->SetPixel( 1, 1, 16, 116 );
	pixel = theVideo->GetPixel( 1, 1, 16 );


//	delete theVideo;
}


void VideoTest_4( void )
{
	typedef unsigned char																			MonoPixel;
	typedef InPlaceAllocatorFor< MonoPixel >														InPlaceAllocatorForMonochrome;
	typedef TVideoFor< MonoPixel, PixelAccess_Trait< MonoPixel >, InPlaceAllocatorForMonochrome >	MonochromeVideoInPlace;

	int imCols = 128;
	int imRows = 256;
	int imImages = 17;

	unsigned long single_image_size = imCols * imRows * sizeof( unsigned char );
	unsigned long buf_size = single_image_size * imImages;
	unsigned char * buf = new unsigned char [ buf_size ];


	// Create the in place allocator
	InPlaceAllocatorForMonochrome inPlaceAllocator( buf );


	typedef SaveBufferAllocator< MonoPixel, InPlaceAllocatorForMonochrome > MonochromeVideo_SafeBufferAllocator_InPlace;
	MonochromeVideo_SafeBufferAllocator_InPlace	theMonochromeVideo_SafeBufferAllocator_InPlace( single_image_size, imImages, inPlaceAllocator );		// it works like auto_ptr

		
		// ##########
		//MonochromeVideo_SafeBufferAllocator_InPlace nextSaveBuffer( theMonochromeVideo_SafeBufferAllocator_InPlace );		this should not compile since the copy constructor is protected, as we wish
		// ##########
	
		MonochromeVideoInPlace monoVid;

		// Create the video
		for( int i = 0; i < imImages; ++ i )
		{
			monoVid.AttachOrphanedFrame( new MonochromeVideoInPlace::FrameImage( imCols, imRows, 0, theMonochromeVideo_SafeBufferAllocator_InPlace.GetNext_InPlaceAllocator() ) );
		}


		monoVid.SetPixel( 1, 1, 1, 125 );
		MonochromeVideo::PixelType pixel = monoVid.GetPixel( 1, 1, 1 );


		pixel = monoVid.GetPixel( 1, 1, 2 );


		monoVid.RemoveFrames( 1, 1 );


	delete [] buf;
}

void VideoTest_5( void )
{
	int imCols = 128;
	int imRows = 256;
	int imImages = 17;

	ColorVideo * theVideo = OrphanColorVideo( imCols, imRows, imImages );

	theVideo->SetPixel( 1, 1, 0, Color_3x8_Pixel( 22, 33, 44 ) );
	ColorVideo::PixelType pixel = theVideo->GetPixel( 1, 1, 0 );
	REQUIRE( pixel == Color_3x8_Pixel( 22, 33, 44 ) );

	pixel = theVideo->GetPixel( 1, 1, 10 );


	delete theVideo;
}


//////////////////////////////////////////////////////////////////////////

void ImageAllocatorTest( void )
{
	int imCols = 128;
	int imRows = 256;

	unsigned long buf_size = imCols * imRows * sizeof( unsigned char );
	unsigned char * buf = new unsigned char [ buf_size ];
	
	typedef InPlaceAllocatorFor< unsigned char >	InPlaceAllocatorForMonochrome;
	
	typedef TImageFor< unsigned char, PixelAccess_Trait< unsigned char >, InPlaceAllocatorForMonochrome >	MonochromeImageInPlace;
	
	// Create the in place allocator
	InPlaceAllocatorForMonochrome inPlaceAllocator( buf/*, buf_size, buf_size*/ );
	

		// @@@@@@@@@@@
		// Testing copy allocators
		InPlaceAllocatorFor< Color_3x8_Pixel > inPlaceAllocatorForColor( inPlaceAllocator );


		// @@@@@@@@@@@


		// Create image with a priori given memory location
		MonochromeImageInPlace * inPlaceImage_1 = new MonochromeImageInPlace( imCols, imRows, inPlaceAllocator );
		

		// ... do something with the image
		inPlaceImage_1->SetPixel( 0, 0, 100 );
		unsigned char pixy_1 = inPlaceImage_1->GetPixel( 0, 0 );


		// and destroy it
		delete inPlaceImage_1;



		// Create a second image in the SAME memory location
		MonochromeImageInPlace * inPlaceImage_2 = new MonochromeImageInPlace( imCols, imRows, inPlaceAllocator );
		

		// ... do something with the image
		inPlaceImage_2->SetPixel( 0, 0, 200 );
		unsigned char pixy_2 = inPlaceImage_2->GetPixel( 0, 0 );


		// and destroy it
		delete inPlaceImage_2;



	// Close the memory buffer
	delete [] buf;
}


//////////////////////////////////////////////////////////////////////////


void HardwareImageLibraryProxyTest( void )
{
	/////////////////////////////////////////
	ImageAllocatorTest();


	/////////////////////////////////////////
	VideoTest();
	VideoTest_2();
	VideoTest_3();
	VideoTest_4();
	VideoTest_5();
	/////////////////////////////////////////

	SG_Filter_Test();

	/////////////////////////////////////////


	BinaryImage theBinaryImage( 10, 10 );

	MaskedMonochromeImage theMaskedImage( 10, 10 );


	MorphologyFor< MonoPixel_Type > theMorphologyObj;

	const int kStructElem_Cols = 3;
	const int kStructElem_Rows = 3;
	const int kStructElem_InitVal = 0;

	MonochromeImage theTestMorphoImage( 10, 10 );

	MaskedMonochromeImage theStructuralElement( kStructElem_Cols, kStructElem_Rows, kStructElem_InitVal, true );		// Create the structural element
	
	MIAP theErodedBinaryImage( theMorphologyObj.Erode( theTestMorphoImage, theStructuralElement ) );


	ImageOperation_AutoPtr erode_ap( Erode_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) );
	( * erode_ap )();

	// create and execute immediately
	( * Dilate_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) )();
	( * Open_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) )();
	( * Close_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) )();
	( * MorphoGradient_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) )();
	( * TopHat_AP( theTestMorphoImage, theTestMorphoImage, theStructuralElement ) )();



	/////////////////////////////////////////


	int size_of = sizeof( MonochromeImage );
	size_of = sizeof( TRealImage );
	size_of = sizeof( ImageForFixed_8_8 );
	size_of = sizeof( ImageForFixed_16_16 );


	FIXED_8_8 fx_test;
	~ fx_test;


	NI_Color_3xFi16_16_Image theColorImage( 77, 113, Color_3xFi16_16_Pixel() );

	int test_val = theColorImage.GetCol();
	test_val = theColorImage.GetRow();

	Color_3xFi16_16_Pixel theColorPixel;

	theColorImage.SetPixel( 76, 22, theColorPixel );

	theColorPixel = theColorImage.GetPixel( 76, 22 );



	NI_Color_3xFi16_16_Image theColorImage2( 77, 113, Color_3xFi16_16_Pixel() );
	NI_Color_3xFi16_16_Image theColorImage3( /*4, 44,*/77, 113, Color_3xFi16_16_Pixel() );
	NI_Color_3xFi16_16_Image theColorImage4( theColorImage3 );
	NI_Color_3xFi8_8_Image theColorImage5( theColorImage3 );

	theColorImage3 = theColorImage2;
	theColorImage5 = theColorImage2;


	try
	{
		theColorImage4 = theColorImage3 + theColorImage2;

		theColorImage3 += theColorImage2;
		theColorImage3 = theColorImage4 + theColorImage2;

		theColorImage3 -= theColorImage2;
		theColorImage3 = theColorImage4 - theColorImage2;

		theColorImage3 *= theColorImage2;
		theColorImage3 = theColorImage4 * theColorImage2;

		theColorImage3 /= theColorImage2;
		theColorImage3 = theColorImage4 / theColorImage2;
	}
	catch( ... )
	{
		std::cerr << "Arithmetic operation error";
	}






	bool are_equal = theColorImage3 == theColorImage;





	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	const int kCols = 25;
	const int kRows = 12;

	ImageForFixed_16_16 i1( kCols, kRows );
	ImageForFixed_16_16 i2( kCols, kRows );
	ImageForFixed_16_16 i3( kCols, kRows );



	i1.SetPixel_Modulo( 1000, 1000, 0 );

	ImageOperation_AutoPtr add_1( Add_AP( i3, i2, i1 ) );
	( * add_1 )();


	ImageOperation_AutoPtr add_2( Add_AP( i3, i2, 4.0 ) );
	( * add_2 )();


	ImageOperation_AutoPtr add_3( Add_AP( i3, i3, 4.0 ) );
	( * add_3 )();

	////////
	ImageOperation_AutoPtr sub_1( Sub_AP( i3, i2, i1 ) );
	( * sub_1 )();
	ImageOperation_AutoPtr sub_2( Sub_AP( i3, i2, 4.0 ) );
	( * sub_2 )();

	////////
	ImageOperation_AutoPtr mul_1( Mul_AP( i3, i2, i1 ) );
	( * mul_1 )();
	ImageOperation_AutoPtr mul_2( Mul_AP( i3, i2, 4.0 ) );
	( * mul_2 )();

	////////
	ImageOperation_AutoPtr div_1( Div_AP( i3, i2, i1 ) );
	try
	{
		( * div_1 )();
	}
	catch(...)
	{
		std::cerr << "Divide by zero";
	}
	ImageOperation_AutoPtr div_2( Div_AP( i3, i2, 4.0 ) );
	try
	{
		( * div_2 )();
	}
	catch(...)
	{
		std::cerr << "Divide by zero";
	}


	////////
	ImageOperation_AutoPtr and_1( AND_AP( i3, i2, i1 ) );
	( * and_1 )();
	ImageOperation_AutoPtr and_2( AND_AP( i3, i2, 4.0 ) );
	( * and_2 )();

	////////
	ImageOperation_AutoPtr or_1( OR_AP( i3, i2, i1 ) );
	( * or_1 )();
	ImageOperation_AutoPtr or_2( OR_AP( i3, i2, 4.0 ) );
	( * or_2 )();

	////////
	ImageOperation_AutoPtr xor_1( XOR_AP( i3, i2, i1 ) );
	( * xor_1 )();
	ImageOperation_AutoPtr xor_2( XOR_AP( i3, i2, 4.0 ) );
	( * xor_2 )();

	////////
	ImageOperation_AutoPtr not_1( NOT_AP( i3, i2 ) );
	( * not_1 )();
	ImageOperation_AutoPtr not_2( NOT_AP( i3, i3 ) );
	( * not_2 )();
	
	
	////////
	ImageOperation_AutoPtr convolve1( _2D_Convolve_AP( i3, i2, i1 ) );
	( * convolve1 )();

// Out since it generates REQUIRES (but for purpose)
//	ImageOperation_AutoPtr horzConv_1( Horz_Convolve_AP( i3, i2, i1 ) );
//	( * horzConv_1 )();	// should fire REQUIRE since the mask contains more rows than one

	FIXED_16_VEC	theHorzMask_Vec;
//	ImageOperation_AutoPtr horzConv_2( Horz_Convolve_AP( i3, i1, theHorzMask_Vec ) );
//	( * horzConv_2 )();	// should fire REQUIRE since the mask is empty

	OpCompletCallback_AutoPtr myCallback( new MyCallBack );
	/*TVectorMultiImageOperation*/TListMultiImageOperation	theCollector( kgThreadSecurity, * myCallback );

	theCollector.AddAdoptNewOperation( Orphan_Add( i3, i2, 3.0 ) );
	theCollector.AddAdoptNewOperation( Orphan_2D_Convolve( i3, i2, i1 ) );
	theCollector.AddAdoptNewOperation( Orphan_Horz_Convolve( i3, i2, i1 ) );
	theCollector.AddAdoptNewOperation( Orphan_Horz_Convolve( i3, i1, theHorzMask_Vec ) );
//	theCollector();		// execute all (some REQUIRES to test)


	////////////////

	i3.SetAll( 5 );
	i3.SetPixel( 7, 8, 6 );
	i3.SetPixel( 9, 10, 4 );
	ImageOperation_AutoPtr findMaxObj( FindMaxVal_AP( i3, i3 ) );
	( * findMaxObj )();		// should fire REQUIRE since we'll be obliterating the input as well?!

	TImageFor< int > retValueForMin( 3, 1 );
	ImageOperation_AutoPtr  findMinObj( FindMinVal_AP( retValueForMin, i3 ) );	// this should be better
	( * findMinObj )();
	cout << "Min value = " << retValueForMin.GetPixel( 0, 0 ) << endl;
	cout << "x = " << retValueForMin.GetPixel( 1, 0 ) << endl;
	cout << "y = " << retValueForMin.GetPixel( 2, 0 ) << endl;
	// The same info but with helpers
	typedef FindMinVal_OperationFor< TImageFor< int >, ImageForFixed_16_16 > * FPTR;
	cout << "Min value = " << ((FPTR)findMinObj.get())->GetMinVal() << endl;
	cout << "x = " << ((FPTR)findMinObj.get())->GetMinVal_Column() << endl;
	cout << "y = " << ((FPTR)findMinObj.get())->GetMinVal_Row() << endl;


	/////////////////

	ImageForFixed_8_8 i5( kCols, kRows );

	ImageOperation_AutoPtr convert_Fixed_8x8_To_16x16( FormatConverter_AP( i3, i5 ) );
	( * convert_Fixed_8x8_To_16x16 )();

	/////////////////

	// Template-image test
	typedef TImageFor< MonochromeImage > MonochromeImageImage;		// an image with pixels being... images

	MonochromeImage aDummyMonoImage( 13, 17 );
	MonochromeImage aDummyMonoImage2( 2, 22 );

	MonochromeImageImage theMonochromeImageImage( 1, 100 );

	theMonochromeImageImage.SetPixel( 0, 0, aDummyMonoImage );		// to many copies
	theMonochromeImageImage.SetRefPixel( 0, 1, aDummyMonoImage2 );	// should be faster

	theMonochromeImageImage.SetRefPixel( 0, 2, theMonochromeImageImage.GetRefPixel( 0, 1 ) );



	/////////////////

}



// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

