
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: HardwareImageLibraryProxy.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				A suite of functions for testing different modules of the
//				library.
//
// ==========================================================================


#ifndef BoxySoft_HardwareImageLibraryProxy
#define BoxySoft_HardwareImageLibraryProxy


#include "HIL_BaseDefinitions.h"
#include "HIL_ArithmeticOperators.h"
#include "HIL_LogicalOperators.h"
#include "HIL_ComposedOperations.h"
#include "HIL_ConvolveOperators.h"
#include "HIL_FindValueInImageOperators.h"
#include "HIL_ImageFormatConvertOperators.h"
#include "HIL_MultiChannelImageFor.h"

#include "ImageErrorMeasurementLibrary.h"



using namespace PHIL;


template< class T >
void TensorTestFor( const MonochromeImage & inImage, TRealImage & Jxx, TRealImage & Jxy, TRealImage & Jyy )
{
	const int kCols = inImage.GetCol();
	const int kRows = inImage.GetRow();

	TImageFor< T > inputImage( kCols, kRows );

	int dummy_test = sizeof( T );


	( * FormatConverter_AP( inputImage, inImage ) )();

	TMultiChannelImageFor< T >	outputImage( kCols, kRows );		// three components of the structural tensor


	// Simoncelli horizontal (vertical) gradient is obtained in a twofold mode:
	// - first we do vertical (horizontal) smoothing with the prefilter
	// - then we apply the horizontal (vertical) derivative filter
	vector< T > theSimoncelliPrefilter;

	theSimoncelliPrefilter.push_back( 0.22420981526374817 );
	theSimoncelliPrefilter.push_back( 0.5515803694725037 );
	theSimoncelliPrefilter.push_back( 0.22420981526374817 );


	vector< T > theSimoncelliDerivative;

	theSimoncelliDerivative.push_back( -0.45527133345603943 );
	theSimoncelliDerivative.push_back( 0.0 );
	theSimoncelliDerivative.push_back( 0.45527133345603943 );


	vector< T > theBinomialSmoothing;

	theBinomialSmoothing.push_back( 0.25 );
	theBinomialSmoothing.push_back( 0.5 );
	theBinomialSmoothing.push_back( 0.25 );



	TImageFor< T > tmpImage_1( kCols, kRows );
	TImageFor< T > tmpImage_2( kCols, kRows );
	TImageFor< T > tmpImage_3( kCols, kRows );


	TVectorMultiImageOperation	* Jxx_Computer = new TVectorMultiImageOperation;
	TVectorMultiImageOperation	* Jxy_Computer = new TVectorMultiImageOperation;
	TVectorMultiImageOperation	* Jyy_Computer = new TVectorMultiImageOperation;

	// Compute the tensor components: Jij = F(Ri*Rj)

	// Precompute the horz derivative (to tmpImage_1)
	Jxx_Computer->AddAdoptNewOperation( Vert_Convolve_AP( tmpImage_3, inputImage, theSimoncelliPrefilter ) );
	Jxx_Computer->AddAdoptNewOperation( Horz_Convolve_AP( tmpImage_1, tmpImage_3, theSimoncelliDerivative ) );

	// Precompute the vert derivative (to tmpImage_2)
	Jxx_Computer->AddAdoptNewOperation( Horz_Convolve_AP( tmpImage_3, inputImage, theSimoncelliPrefilter ) );
	Jxx_Computer->AddAdoptNewOperation( Vert_Convolve_AP( tmpImage_2, tmpImage_3, theSimoncelliDerivative ) );



	// Jxx
	Jxx_Computer->AddAdoptNewOperation( Orphan_Mul( outputImage.GetRefPixel( 0, 0 ), tmpImage_1, tmpImage_1 ) );
	Jxx_Computer->AddAdoptNewOperation( Orphan_Vert_Convolve( tmpImage_3, outputImage.GetRefPixel( 0, 0 ), theBinomialSmoothing ) );
	Jxx_Computer->AddAdoptNewOperation( Orphan_Horz_Convolve( outputImage.GetRefPixel( 0, 0 ), tmpImage_3, theBinomialSmoothing ) );


	// Jxy
	Jxy_Computer->AddAdoptNewOperation( Orphan_Mul( outputImage.GetRefPixel( 1, 0 ), tmpImage_1, tmpImage_2 ) );
	Jxy_Computer->AddAdoptNewOperation( Orphan_Vert_Convolve( tmpImage_3, outputImage.GetRefPixel( 1, 0 ), theBinomialSmoothing ) );
	Jxy_Computer->AddAdoptNewOperation( Orphan_Horz_Convolve( outputImage.GetRefPixel( 1, 0 ), tmpImage_3, theBinomialSmoothing ) );

	// Jyy
	Jyy_Computer->AddAdoptNewOperation( Orphan_Mul( outputImage.GetRefPixel( 2, 0 ), tmpImage_2, tmpImage_2 ) );
	Jyy_Computer->AddAdoptNewOperation( Orphan_Vert_Convolve( tmpImage_3, outputImage.GetRefPixel( 2, 0 ), theBinomialSmoothing ) );
	Jyy_Computer->AddAdoptNewOperation( Orphan_Horz_Convolve( outputImage.GetRefPixel( 2, 0 ), tmpImage_3, theBinomialSmoothing ) );



	TVectorMultiImageOperation	compoundComputer;		// this object can be automatic
	compoundComputer.AddAdoptNewOperation( Jxx_Computer );
	compoundComputer.AddAdoptNewOperation( Jxy_Computer );
	compoundComputer.AddAdoptNewOperation( Jyy_Computer );

	// Compute all
	compoundComputer();


	// Prepare output image
	( * FormatConverter_AP( Jxx, outputImage.GetRefPixel( 0, 0 ) ) ) ();
	( * FormatConverter_AP( Jxy, outputImage.GetRefPixel( 1, 0 ) ) ) ();
	( * FormatConverter_AP( Jyy, outputImage.GetRefPixel( 2, 0 ) ) ) ();

}



#endif BoxySoft_HardwareImageLibraryProxy




