
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: ImageProcessingTest.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Test functions. If using, plese be sure to change
//				the hard-coded path names.
//
// ==========================================================================


#pragma warning(disable:4786)

#include <stdio.h>
#include "HardwareImageLibraryProxy.h"
#include "fstream"
#include "HIL_Exceptions.h"
#include "HIL_RankFilterOperators.h"
#include "HIL_ThresholdingOperators.h"
#include "NonParametricTransformations.h"
#include "HIL_NonParamConvertOperators.h"
#include "HIL_PyramidOperators.h"
#include "HIL_HistogramOperators.h"
#include "HIL_WarpingOperators.h"
#include "Image_IO.h"
#include "Moments.h"
#include "AffineMoments.h"


using namespace PHIL;


////////////////////////////////////////////////////////////////////////////////////////////////






///////////////////////////////////////////////////////////
// This function 
///////////////////////////////////////////////////////////
//		
// INPUT:
//		
//		
// OUTPUT:
//		
//		
// REMARKS:
//		
//		
void StatisticalMoments_Test( void )
{

	MonochromeImage image_1( 256, 256, 255 );

	MaskedMonochromeImage masked_image_1( image_1 );

	double intertia = InertiaMoment( masked_image_1 );

	Real_2D_Point theCentroid = CenterOfGravity( masked_image_1 );

	vector< double > theAffineInvariants;
	bool operation_result = AffineMomentInvariants( masked_image_1, theCentroid, theAffineInvariants );

}


///////////////////////////////////////////////////////////
// This function computes correlation between images.
// It is a simple version of SAD metric.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void ImageCorrelation_Test( void )
{
	// Load the input images
	MonochromeImage image_1( 256, 256 );
	if( LoadImage( image_1, "artkor1.raw" ) == false )
		return;

	MonochromeImage image_2( 256, 256 );
	if( LoadImage( image_2, "artkor2.raw" ) == false )
		return;

	LongImage tmpImage( image_1 );

	( * Sub_AP( tmpImage, tmpImage, image_2 ) )();	// find differences

	( * Abs_AP( tmpImage, tmpImage ) )();			// find abs of differences

	MonochromeImage image_3( tmpImage );
	if( SaveImage( image_3, "artkor_correlation.raw" ) == false )
		return;

}



///////////////////////////////////////////////////////////
// This function computes image histograms.
// Then histograms are converted to a visualized image
// and saved.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void ImageHistograms_Test( void )
{
	// Load the input image
	MonochromeImage testInImage( 256, 256 );
	if( LoadImage( testInImage, "artkor1.raw" ) == false )
		return;


	THistogram::HistoVector_AP histogram;

	( * ImageHistogram_AP( histogram, testInImage ) )();	// initialize and run the histogram operator


	// Let's prepare the visualization image
	const MonochromeImage::PixelType kMaxVal = 255;
	const MonochromeImage::PixelType kMinVal = 0;

	REQUIRE( histogram->size() == 256 );
	MonochromeImage histogramImage( histogram->size(), histogram->size(), kMaxVal );

	// Normalize the histogram, so it can fit into the visualization image
	typedef THistogram::HistoVector::iterator HistoVectorIterator;
	/*unsigned int **/HistoVectorIterator maxHistogramValuePos = max_element( histogram->begin(), histogram->end() );
	//REQUIRE( maxHistogramValuePos != 0 );
	REQUIRE( * maxHistogramValuePos != 0 );		// at least something had to be found ?
	transform( histogram->begin(), histogram->end(), histogram->begin(), bind2nd( divides< double >(), ((double)( * maxHistogramValuePos ))/(double)histogram->size() ) );


	// Fill the histogram image to visualize the histogram.
	for( int i = 0; i < histogramImage.GetCol(); ++ i )
	{
		int theHistCounter = (*histogram)[ i ];
		int theRowCounter = histogramImage.GetRow();
		
		while( theHistCounter -- )
		{
			REQUIRE( theRowCounter > 0 );
			histogramImage.SetPixel( i, -- theRowCounter, kMinVal );
		}
	}

	// Save results
	if( SaveImage( histogramImage, "artkor1_hist.raw" ) == false )
		return;

}

///////////////////////////////////////////////////////////
// This function saves a given template-image as a series
// of images with the core name given by baseFileName.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			theImage - reference to the template-image
//				to be stored; it is assumed that this 
//				is a one-row image of images
//			baseFileName - base name that is modified
//				by adding a successive number just before
//				the last '.' found in it
//		
// OUTPUT:
//			true if operation successfull
//			false otherwise
//		
// REMARKS:
//		
//		
template < class T >
bool SaveTemplateImage( const TImageFor< TImageFor< T > > & theImage, const char * baseFileName )
{
	string str( baseFileName );

	char next_value = '0';

	string::size_type point_index = str.find( '.' );
	if( point_index == string::npos )
		point_index = str.length();

	const int kCols = theImage.GetCol();
	REQUIRE( theImage.GetRow() > 0 );
	if( theImage.GetRow() == 0 )
		return false;
	
	for( int i = 0; i < kCols; ++ i )
	{
		string fileName( str );
		fileName.insert( point_index, 1, next_value ++ );	// modify an output file name a little
		 
		TImageFor< T > tmpImage( theImage.GetRefPixel( i, 0 ) );
		MonochromeImage tmpMonoImage( tmpImage.GetCol(), tmpImage.GetRow() );
		ChangeImageRange( tmpImage, (T)0, (T)255 );
		MixedCopy( tmpMonoImage, tmpImage );
		if( SaveImage( tmpMonoImage, fileName.c_str() ) == false )
			return false;
	}

	return true;
}

///////////////////////////////////////////////////////////
// This function computes image pyramids.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void ImagePyramids_Test( void )
{

	MonochromeImage testInImage( 256, 256 );
	if( LoadImage( testInImage, "artkor1.raw" ) == false )
		return;

	TRealImage testRealInImage( 256, 256 );

	ImageAbsoluteValue( testRealInImage );

	MixedCopy( testRealInImage, testInImage );

	int pyramidLevels = 5;
	int maskWidth = 3;
	double sigma = 2;

	RealImage_Vector theImageVector;

	( * Gaussian_Pyramid_AP( theImageVector, testRealInImage, pyramidLevels, maskWidth, sigma ) )();

	( * Laplacian_Pyramid_AP( theImageVector, testRealInImage, pyramidLevels, maskWidth, sigma ) )();


	// Now the same but with the template-image as an output container 

	Real_Template_Image theOutputTemplateImage( pyramidLevels, 1 );

	( * Gaussian_Pyramid_AP( theOutputTemplateImage, testRealInImage, pyramidLevels, maskWidth, sigma ) )();

	// Save the template image
	SaveTemplateImage( theOutputTemplateImage, "artkor_gauss_pyr_.raw" );

	( * Laplacian_Pyramid_AP( theOutputTemplateImage, testRealInImage, pyramidLevels, maskWidth, sigma ) )();

	// Save the template image
	SaveTemplateImage( theOutputTemplateImage, "artkor_lapl_pyr_.raw" );


}


////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////
// This function computes non-parametric (Census)
// transform of images.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void NonParam_Test( void )
{
	MonochromeImage testInImage( 256, 256 );
	if( LoadImage( testInImage, "artkor1.raw" ) == false )
		return;

	MonochromeImage outImage( 256, 256 );


	////////////

	
	( * Census3x3_Converter_AP( outImage, testInImage ) )();
	
	
	Census_3x3::_NonParamImage outImage_2( testInImage.GetCol(), testInImage.GetRow() );
	( * Census3x3_Converter_AP( outImage_2, testInImage ) )();



	( * Census5x5_Converter_AP( outImage, testInImage ) )();


}



///////////////////////////////////////////////////////////
// This function computes image thresholds.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void ThresholdTest( void )
{
	MonochromeImage in( 3, 3, 0 );
	MonochromeImage out( 3, 3, 0 );

	( * ImageThreshold_AP( out, in, 100, 120, 0, 1, 2 ) )();
}



///////////////////////////////////////////////////////////
// This function computes median output.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void MedianFilterTest( void )
{
	MonochromeImage in( 3, 3, 0 );
	MonochromeImage out( 3, 3, 0 );

	in.SetPixel( 0, 0, 1 );
	in.SetPixel( 1, 0, 2 );
	in.SetPixel( 2, 0, 3 );
	in.SetPixel( 0, 1, 4 );
	in.SetPixel( 1, 1, 5 );
	in.SetPixel( 2, 1, 6 );
	in.SetPixel( 0, 2, 7 );
	in.SetPixel( 1, 2, 8 );
	in.SetPixel( 2, 2, 9 );

	MedianFilter( in, 3, 3, out );


	int median = out.GetPixel( 1, 1 );

	cout << median;

	// Try the new HIL operators for median filtering
	MonochromeImage neighborhood( 1, 2 );
	neighborhood.SetPixel( 0, 0, 3 );
	neighborhood.SetPixel( 0, 1, 3 );
	( * Median_Filter_AP( out, in, neighborhood ) )();

	cout << endl << (int)out.GetPixel( 1, 1 );		// output the median

	Index mask_size( 3, 3 );
	( * Median_Filter_AP( out, in, mask_size ) )();

	cout << endl << (int)out.GetPixel( 1, 1 );		// output the median
}

///////////////////////////////////////////////////////////
// This function computes median from non-scalar pixel
// images.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void ColourMedianFilterTest( void )
{

	ColorImage in( 13, 13, Color_3x8_Pixel( (unsigned char)0 ) );
	ColorImage out( 13, 13, Color_3x8_Pixel( (unsigned char)0 ) );

	in.SetPixel( 0, 0, Color_3x8_Pixel( 1 ) );
	in.SetPixel( 1, 0, Color_3x8_Pixel( 2 ) );
	in.SetPixel( 2, 0, Color_3x8_Pixel( 3 ) );
	in.SetPixel( 0, 1, Color_3x8_Pixel( 4 ) );
	in.SetPixel( 1, 1, Color_3x8_Pixel( 5 ) );
	in.SetPixel( 2, 1, Color_3x8_Pixel( 6 ) );
	in.SetPixel( 0, 2, Color_3x8_Pixel( 7 ) );
	in.SetPixel( 1, 2, Color_3x8_Pixel( 8 ) );
	in.SetPixel( 2, 2, Color_3x8_Pixel( 9 ) );


	MedianFilter( in, 3, 3, out );


	Color_3x8_Pixel median = out.GetPixel( 1, 1 );

	// Since "median" is a mulit pixel with components of type
	// "unsigned char" it will not print properly on "cout".
	// Therefore we have to convert to some other pixel 
	// (Int_3D_Point in our case) and do <<
	cout << (Int_3D_Point)median;

	// Try the new HIL operators for median filtering
	MonochromeImage neighborhood( 1, 2 );
	neighborhood.SetPixel( 0, 0, 3 );
	neighborhood.SetPixel( 0, 1, 3 );
	( * Median_Filter_AP( out, in, neighborhood ) )();

	cout << endl << (Int_3D_Point)out.GetPixel( 1, 1 );		// output the median

	Index mask_size( 3, 3 );
	( * Median_Filter_AP( out, in, mask_size ) )();

	cout << endl << (Int_3D_Point)out.GetPixel( 1, 1 );		// output the median

}



///////////////////////////////////////////////////////////
// This function contains examples of using 
// the MMultiPixelFor<> template class.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void MultiPixel_Test( void )
{
	static unsigned char vArray1[] = { 1, 2, 3 };
	static unsigned char vArray2[] = { 5, 6, 7 };

	Color_3x8_Pixel p1( vArray1 );
	Color_3x8_Pixel p2( vArray2 );
	Color_3x8_Pixel p22( p2 );

	Color_Real_Pixel p3( p22 );
	MMultiPixelFor< double, 5 > p4( 2.0 );
	MMultiPixelFor< long, 5 > p5;


	////////////
	double first_coord = p4[ 0 ];
	double sec_coord = p4[ 1 ];

	p4[ 0 ] = p4[ 1 ] = -3.0;

	first_coord = p4[ 0 ];
	sec_coord = p4[ 1 ];
	///////////


	p22 = p2;
	p3 = p2;
	p5 = p4;

	p3 = p1 + p2;
	p3 = p1 * 2 + p2 * 3;
	p3 = p1 * p2 + 5;

	p3 = p1 + p2 * 2;
//	p3 = p1 + 2 * p2;	error
	p3 = p1 + (Color_3x8_Pixel)2 * p2;	

	p4 = (MMultiPixelFor< double, 5 >)2.0;

	p4 = (MMultiPixelFor< long, 5 >)2 * p5 * 2 + 7;


	double product = InnerProduct( p3, p2 );	

	// result goes to "unsigned char" which is not the best choice but is here to test
	_InnerProduct< double, 3, unsigned char, unsigned char > prod;
	product = prod( p3, p2 );

	// In-Out
	// Open in text mode (default if no ios::binary)
	ofstream test1( "MultiPixel.txt", ios::out );
	if( ! test1.is_open() || test1.fail() )
		return;

	test1 << p1;
	test1 << p2;
	test1 << p3;

	test1.close();

	Color_Real_Pixel p33;

	ifstream test2( "MultiPixel.txt", ios::in );
	if( ! test2.is_open() || test2.fail() )
		return;

	test2 >> p1;
	test2 >> p2;
	test2 >> p33;

	cout << ( p3 == p33 ? "Equal" : "Not equal ?" );



}

///////////////////////////////////////////////////////////
// This function contains examples of global operations
// on images - findig minimal and maximal value in an image.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			none
//		
// REMARKS:
//		
//		
void GlobalOperations_Test( void )
{
	const int kCols = 128;
	const int kRows = 100;

	MonochromeImage theMonoImage( kCols, kRows, 0 );
	MonochromeImage theRealImage( kCols, kRows );

	theMonoImage.SetPixel( 3, 4, 1 );
	theMonoImage.SetPixel( 3, 5, 5 );
	theMonoImage.SetPixel( 3, 6, 7 );



	double test;
	Index pos;

	test = GetImageMinValue( theMonoImage );
	test = GetImageMinValue( theMonoImage, pos );
	test = GetImageMaxValue( theMonoImage );
	test = GetImageMaxValue( theMonoImage, pos );

	MultiThresholding( theMonoImage, (unsigned char)100, (unsigned char)128, theMonoImage, (unsigned char)0, (unsigned char)1, (unsigned char)2 );
	MultiThresholding( theMonoImage, (unsigned char)100, theMonoImage, (unsigned char)0, (unsigned char)1 );

	try
	{
		test = GetImageMeanValue( theMonoImage );
		unsigned long consider_pixels_counter;
		test = GetImageMeanValue( theMonoImage, theMonoImage, 111, consider_pixels_counter );
	}
	catch( ... )
	{
		cerr << "Problems when computing mean value. Empty image?";
	}

}


void WarpTest_1() 
{
	MonochromeImage testInImage( 256, 256 );
	if( LoadImage( testInImage, "artkor1.raw" ) == false )
		return;


	const double rotationInDegs = 22;
	const Real_2D_Point centerOfRotation( 12, 33 );
	const Real_2D_Point translationVector( -30, 0 );
	const Real_2D_Point scalingVector( 1, 1 );

	MonochromeImage outImage( testInImage.GetCol(), testInImage.GetRow() );
	
	( * ImageAffineWarping_AP( outImage, testInImage, rotationInDegs, centerOfRotation, translationVector, scalingVector ) )();


	if( SaveImage( outImage, "artkor1warp.raw" ) == false )
		return;
}

#if CONNECT_2_OPEN_CV

void ColourWarpTest_1() 
{
	ColorImage * testInImage = Orphan_ColorImageFrom( "fruits.TIF" );
	if( testInImage == 0 )
		return;

	// For warping we need better precision
	ColorRealValuedImage testInRealImage( testInImage->GetCol(), testInImage->GetRow() );
	MixedCopy( testInRealImage, * testInImage );

	const double rotationInDegs = 22;
	const Real_2D_Point centerOfRotation( 10, 100 );
	const Real_2D_Point translationVector( 10, 0 );
	const Real_2D_Point scalingVector( 1, 1 );

	ColorRealValuedImage outRealImage( testInImage->GetCol(), testInImage->GetRow(), 0 );
	
	( * ImageAffineWarping_AP( outRealImage, testInRealImage, rotationInDegs, centerOfRotation, translationVector, scalingVector ) )();

	ColorImage outImage( testInImage->GetCol(), testInImage->GetRow() );
	MixedCopy( outImage, outRealImage );


	bool result = Save_ColorImageIn( outImage, "fruits_warp.TIF" );
	REQUIRE( result == true );

}
#endif CONNECT_2_OPEN_CV



void WarpTest_2() 
{
	int cols = 128;
	int rows = 128;

	TRealImage inImage( cols, rows, 0.0 );


	TCompound_LinearTransformEngine coordTransformEngine;

	Int_2D_Point rotationCenterPoint( cols / 2, rows / 2 );
	double rotationAngleDegs = 22;
	TCoordRotationEngine		rotation( rotationAngleDegs / 180.0 * kPi, rotationCenterPoint );
	coordTransformEngine.AddNewTransformation( rotation );

	double tx = 5;
	double ty = 12;
	TCoordTranslationEngine		translation( Real_2D_Point( tx, ty ) );
	coordTransformEngine.AddNewTransformation( translation );

	double sx = 0.9;
	double sy = 1.1;
	TCoordScalingEngine			scaling( Real_2D_Point( sx, sy ) );
	coordTransformEngine.AddNewTransformation( scaling );


	TGenericTransformEngine genericTransformEngine( cols, rows );
	
	// This experiment is as follows: let's take a coordinate-transformation object
	// that describes a given transformation. Then prepare the look-up-image, i.e.
	// the generic transformation.
	for( int i = 0; i < rows; ++ i )
	{
		for( int j = 0; j < cols; ++ j )
		{
			genericTransformEngine.Get_ImageOf_Real_2D_Point().SetPixel( j, i, coordTransformEngine( Real_2D_Point( j, i ) ) );
		}
	}



	TBicubicPolyInterpolation< double >	bicubicPolyInterpolation( inImage );
	TImageBackwardWarp< double > simpleWarpEngine( inImage, inImage.GetCol(), inImage.GetRow(), genericTransformEngine, bicubicPolyInterpolation );

	

	RIAP outImage( simpleWarpEngine() );

	if( outImage.get() == 0 )
		return;

	MonochromeImage displayImage( cols, rows, 0 );

	MixedCopy( displayImage, * outImage );

}


#if CONNECT_2_OPEN_CV

void CommonWarpTest() 
{
	NI_Color_3x8_Image * im = Orphan_NonInterColorImageFrom( "fruits.TIF" );
	bool retVal = Save_ColorImageIn( * im, "fruits2.TIF" );


	//////////////////////

	WarpTest_1();
	WarpTest_2();				// generic warp
	ColourWarpTest_1();
	//ColourWarpTest_2();
}

#endif CONNECT_2_OPEN_CV





///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////






