
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: PixelAccessTraits.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Pixel access traits.	
//				If you define a new type of pixel and/or image then
//				it can be necessary to add a specialization for that type.
//
// ==========================================================================



#ifndef BoxySoft_HIL_PixelAccessTraits
#define BoxySoft_HIL_PixelAccessTraits



// ==========================================================================

namespace PHIL	
{


// The PixelAccess_Trait traits are used by the TImageFor<> suite 
// to access pixels of the types given in specifications.
// They resolve question of how to access pixels in such functions
// as GetPixel() or SetPixel() where pixels can be accessed either
// by value or by reference. This can be defined providing
// specific trait for a given (built-in or user defined) types
// of pixels.


// This trait defines a type which
// will be used when accessing pixels 
template < typename T >
class PixelAccess_Trait
{
	public: 
		// For all undefined access pixels by value
		typedef			T		PixelAccessType;
		typedef const	T		ConstPixelAccessType;
};


// Specific traits are implemented as specializations.
template <> class PixelAccess_Trait< double >
{
	public: 
		// For double access pixels by reference
		typedef			double &	PixelAccessType;
		typedef const	double &	ConstPixelAccessType;
};

template <> class PixelAccess_Trait< int >
{
	public: 
		// For double access pixels by value
		typedef			int		PixelAccessType;
		typedef const	int		ConstPixelAccessType;
};




// ==========================================================================
}	// end of the PHIL namespace
// ==========================================================================



#endif BoxySoft_HIL_PixelAccessTraits




