
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: HIL_HistogramOperators.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Definition of operators for computation of histograms.
//
// ==========================================================================


#ifndef BoxySoft_HIL_HistogramOperators
#define BoxySoft_HIL_HistogramOperators


#include "HIL_BaseOperations.h"
#include "Histogram.h"



namespace PHIL		
{



// ==========================================================================

///////////////////////////////////////////////////////////
//
// This class implements unary image operation:
// image global thresholding.
//
///////////////////////////////////////////////////////////
template< typename RetIm_Type, typename InIm_Type1 >
class Histogram_OperationFor : public TUnaryImageOperationFor< RetIm_Type, InIm_Type1 >
{


	public:

		///////////////////////////////////////////////////////////
		// Class constructor.
		// Histograming is done by comparing each input pixel
		// with thresholds. Then the output value is set based
		// on this comparison, as follows:
		//
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			retImage - reference to the output image
		//				of type RetIm_Type (specified by the 
		//				first template parameter). It should be
		//				able to receive three pixels: one
		//				with the found value and two with pixel
		//				position coordinates.
		//			inImage1 - constant reference to the constant
		//				input image of type InIm_Type1 (specified  
		//				by the second template parameter)
		//			resourceAccessPolicy - optional reference to
		//				the thread security object (derivative
		//				of the TThreadSecurity class); by default
		//				the static kgThreadSecurity object is supplied
		//				which does nothing
		//			opCompCallback - optional reference to the callback
		//				object which is called upon completion of operation;
		//				by default the static kgOperationCompletionCallback
		//				object is supplied which does nothing
		//		
		Histogram_OperationFor( RetType & retImage, const InType_1 const & inImage1, TThreadSecurity & resourceAccessPolicy = kgThreadSecurity, TOperationCompletionCallback & opCompCallback = kgOperationCompletionCallback )
			:	TUnaryImageOperationFor< RetIm_Type, InIm_Type1 >( retImage, inImage1, resourceAccessPolicy, opCompCallback )
				{}

		///////////////////////////////////////////////////////////
		// Class virtual destructor
		///////////////////////////////////////////////////////////
		virtual ~Histogram_OperationFor() {}


		///////////////////////////////////////////////////////////
		// The function operator which finds a pixel with maximal
		// value in an input image, as well as its position in 
		// that image.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			a pointer to the return image;
		//		
		// REMARKS:
		//		
		virtual void * operator()( void )
		{
			MImageOperationRetinue	theImageOperationRetinue( * this );		// create this variable for proper preamble

			THistogram theHistogramBuilder;
			fRetImageRef = theHistogramBuilder.FindHistogram( fInImage1_Ref );
			
			return & fRetImageRef;
		}

};



// ==========================================================================

// Useful helpers:
template< typename RetIm_Type, typename InIm_Type1 >
Histogram_OperationFor< RetIm_Type, InIm_Type1 > * Orphan_ImageHistogram( RetIm_Type & retImage, const InIm_Type1 const & inImage1, TThreadSecurity & resourceAccessPolicy = kgThreadSecurity, TOperationCompletionCallback & opCompCallback = kgOperationCompletionCallback )
{
	return new Histogram_OperationFor< RetIm_Type, InIm_Type1 >( retImage, inImage1, resourceAccessPolicy, opCompCallback );
}


// Yet another helpers:
template< typename RetIm_Type, typename InIm_Type1 >
ImageOperation_AutoPtr ImageHistogram_AP( RetIm_Type & retImage, const InIm_Type1 const & inImage1, TThreadSecurity & resourceAccessPolicy = kgThreadSecurity, TOperationCompletionCallback & opCompCallback = kgOperationCompletionCallback )
{
	return ImageOperation_AutoPtr( new Histogram_OperationFor< RetIm_Type, InIm_Type1 >( retImage, inImage1, resourceAccessPolicy, opCompCallback ) );
}



// ==========================================================================
}	// end of the PHIL namespace

// ==========================================================================


#endif BoxySoft_HIL_HistogramOperators




