
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: ThreadSecurity.h $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Definition of the TThreadSecurity class.
//
// ==========================================================================



#ifndef BoxySoft_TThreadSecurity
#define BoxySoft_TThreadSecurity



///////////////////////////////////////////////////////////
// This class 
///////////////////////////////////////////////////////////
class TThreadSecurity
{
	private:

		// Class inherent variables

	protected:

	public:

		// ===================================================
		TThreadSecurity( void ) {}		// class default constructor
		virtual ~TThreadSecurity() {}	// class virtual destructor
		// ===================================================


	public:

		///////////////////////////////////////////////////////////
		// Override this function in a derived class if thread
		// synchronization is necessary. It should initialize
		// all the necessary means of the thread security.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if access gained,
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual bool Initialize( void ) { return true; /* do nothing in the base class */ }

		///////////////////////////////////////////////////////////
		// Override this function in a derived class if thread
		// synchronization is necessary. It should gain an 
		// access to the critical section
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if access gained,
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual bool Enter_CriticalSection( void ) { return true; /* do nothing in the base class */ }
		
		///////////////////////////////////////////////////////////
		// Override this function in a derived class if thread
		// synchronization is necessary. It should release an 
		// access to the critical section
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if operation ok,
		//			false - there are some errors !!!
		//		
		// REMARKS:
		//		
		//		
		virtual bool Exit_CriticalSection( void ) { return true; /* do nothing in the base class */ }
		
		
};




#endif BoxySoft_TThreadSecurity




