
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: main.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Thread security classes for Win32.	
//
// ==========================================================================


#include "stdafx.h"
#include "Win32_ThreadSecurity.h"



///////////////////////////////////////////////////////////
// The class constructor
///////////////////////////////////////////////////////////
//		
// INPUT:
//			waitTimeMilliseconds - time out value in system ticks
//				for the semaphore pending. If INFINITE 
//				then a thread waits indefinite (default)
//				(value in milliseconds)
// OUTPUT:
//		
// REMARKS:
//		
//		
Win32_ThreadSecurity::Win32_ThreadSecurity( DWORD waitTimeMilliseconds /*= INFINITE*/ )		// class constructor
: fWaitTime( waitTimeMilliseconds ), fSingleLockObj( 0 )
{
}

///////////////////////////////////////////////////////////
// This function 
///////////////////////////////////////////////////////////
//		
// INPUT:
//		
//		
// OUTPUT:
//		
//		
// REMARKS:
//		
//		
Win32_ThreadSecurity::~Win32_ThreadSecurity()		// class destructor
{
	delete fSingleLockObj;
}

///////////////////////////////////////////////////////////
// It should initialize
// all the necessary means of the thread security.
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			true - if access gained,
//			false - otherwise
//		
// REMARKS:
//		
//		
bool Win32_ThreadSecurity::Initialize( void ) 
{ 
	// get rid of the old object
	delete fSingleLockObj;				

	// create and initially do not attempt to access the fCriticalSection object
	fSingleLockObj = new CSingleLock( & fCriticalSection, FALSE );	

	return true; 	
}

///////////////////////////////////////////////////////////
// It should gain an access to the critical section
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			true - if access gained,
//			false - otherwise
//		
// REMARKS:
//		
//		
bool Win32_ThreadSecurity::Enter_CriticalSection( void ) 
{ 
	REQUIRE( fSingleLockObj != 0 );
	// Since this code is called over and over again, then we allow
	// for the resource to be locked at the entry. In the latter,
	// we return false informing that the resource is not available
	// because it has been already locked.
	if( fSingleLockObj == 0 || fSingleLockObj->IsLocked() == TRUE )
		return false;

	// Try to lock the critical resource; wait specified time
	fSingleLockObj->Lock( fWaitTime );	

	// Return "true" if we have gained an access to that resource
	return fSingleLockObj->IsLocked() == TRUE;
}

///////////////////////////////////////////////////////////
// It should release an 
// access to the critical section
///////////////////////////////////////////////////////////
//		
// INPUT:
//			none
//		
// OUTPUT:
//			true - if operation ok,
//			false - there are some errors !!!
//		
// REMARKS:
//		
//		
bool Win32_ThreadSecurity::Exit_CriticalSection( void ) 
{ 
	REQUIRE( fSingleLockObj != 0 )
	if( fSingleLockObj == 0 )
		return false;

	// Try to lock the critical resource; wait specified time
	BOOL unlockResult = fSingleLockObj->Unlock();	
	REQUIRE( unlockResult == TRUE );
	return unlockResult == TRUE;
}

