
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: main.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Thread security classes for Win32.	
//
// ==========================================================================


#ifndef BoxySoft_Win32_ThreadSecurity
#define BoxySoft_Win32_ThreadSecurity



#include "ThreadSecurity.h"			// for OS synchronization objects
#include <afxmt.h>

class CSingleLock;


///////////////////////////////////////////////////////////
// This class 
///////////////////////////////////////////////////////////
class Win32_ThreadSecurity : public TThreadSecurity
{
	private:

		// Class inherent variables

		CSingleLock * fSingleLockObj;

	protected:

		CCriticalSection	fCriticalSection;		// for multi-threaded access
													// to the buffer
		// Wait for the synchronization object 
		// to become signaled up to the number of milliseconds. 
		// Can be also INFINITE.
		DWORD				fWaitTime;				

	public:

		// ===================================================
		
		///////////////////////////////////////////////////////////
		// The class constructor
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			waitTimeMilliseconds - time out value in system ticks
		//				for the semaphore pending. If INFINITE 
		//				then a thread waits indefinite (default)
		//				(value in milliseconds)
		//		
		// OUTPUT:
		//		
		// REMARKS:
		//		
		//		
		Win32_ThreadSecurity( DWORD waitTimeMilliseconds = INFINITE );		// class default constructor
		
		virtual ~Win32_ThreadSecurity();									// class virtual destructor
		
		// ===================================================


	public:


		///////////////////////////////////////////////////////////
		// It should initialize
		// all the necessary means of the thread security.
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if access gained,
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual bool Initialize( void );

		///////////////////////////////////////////////////////////
		// It should gain an access to the critical section
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if access gained,
		//			false - otherwise
		//		
		// REMARKS:
		//		
		//		
		virtual bool Enter_CriticalSection( void );
		
		///////////////////////////////////////////////////////////
		// It should release an 
		// access to the critical section
		///////////////////////////////////////////////////////////
		//		
		// INPUT:
		//			none
		//		
		// OUTPUT:
		//			true - if operation ok,
		//			false - there are some errors !!!
		//		
		// REMARKS:
		//		
		//		
		virtual bool Exit_CriticalSection( void );		
		


};



#endif BoxySoft_Win32_ThreadSecurity




