
// ==========================================================================
//
// Software written by Boguslaw Cyganek (C) 2008
// Copyrights Pandora Inc. (C) 2008
//
// ==========================================================================
//
// This software is protected by the national and international copyrights. 
// The software is granted only for educational or academic/research purposes   
// and to accompany the book (credited to):
// "AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS"
// by Boguslaw Cyganek & J. Paul Siebert, Wiley, 2008.
// All other commercial and/or non-private applications require a written 
// permission.
// The software is supplied as it is without any guarantees nor responsibility  
// of its use in any application. 
//
// ==========================================================================
//
//  Background: AN INTRODUCTION TO 3D COMPUTER VISION TECHNIQUES AND ALGORITHMS
//  Version:    1.0
//
//  $Workfile: main.cpp $
//  $Author  : Boguslaw Cyganek
//  $Created : 2007-05-06 21:32:58
//
//
//  Description:
//				Contains the main function which launches simple functions
//				which test functionality of the library.	
//
// ==========================================================================



#pragma warning(disable:4786)

#include <stdio.h>
#include "HardwareImageLibraryProxy.h"
#include "fstream"
#include "HIL_Exceptions.h"
#include "HIL_RankFilterOperators.h"
#include "HIL_ThresholdingOperators.h"
#include "NonParametricTransformations.h"
#include "HIL_NonParamConvertOperators.h"
#include "HIL_PyramidOperators.h"
#include "HIL_HistogramOperators.h"
#include "IntelLIb_ImageConverters.h"			// for definition of the: CONNECT_2_OPEN_CV


using namespace PHIL;



extern void HardwareImageLibraryProxyTest( void );
extern void Mono_IPL_Test( const char * );
extern void NonInter_Color_IPL_Test( const char * );
extern void Inter_Color_IPL_Test( const char * );
extern void ImageCorrelation_Test( void );
extern void ImageHistograms_Test( void );
extern void ImagePyramids_Test( void );
extern void NonParam_Test( void );
extern void ThresholdTest( void );
extern void MedianFilterTest( void );
extern void ColourMedianFilterTest( void );
extern void GlobalOperations_Test( void );
extern void MultiPixel_Test( void );
extern void CommonWarpTest() ;
extern void StatisticalMoments_Test() ;






int main(int argc, char* argv[])
{

	try
	{
		///////////////////
		// Flat tensor test
		void FlatTensorTest( void );
		FlatTensorTest();
		///////////////////


#if CONNECT_2_OPEN_CV
		CommonWarpTest();
#endif CONNECT_2_OPEN_CV

		StatisticalMoments_Test();


		ImageCorrelation_Test();


		ImageHistograms_Test();


		ImagePyramids_Test();


		NonParam_Test();


		ThresholdTest();


		MedianFilterTest();

		ColourMedianFilterTest();


		GlobalOperations_Test();

		const int kCols = 128;
		const int kRows = 100;

		MonochromeImage theMonoImage( kCols, kRows );
		TRealImage Jxx( kCols, kRows );
		TRealImage Jxy( kCols, kRows );
		TRealImage Jyy( kCols, kRows );


		TensorTestFor< FIXED_16_16 >( theMonoImage, Jxx, Jxy, Jyy );

		HardwareImageLibraryProxyTest();


		MultiPixel_Test();

	}
	catch( T_Standard_HIL_Exception & e )
	{
		cout << e;
	}
	catch( ... )
	{
		cout << "What is that expression?";
	}


	cout << endl << "Press 0 to exit" << endl;
	
	int dummy;
	cin >> dummy;		// to stop the console window

	return 0;
}

